@@ -3,25 +3,21 @@
 # Purpose: Provides python autocompletion lists and calltips for the editor   #
 # Author: Cody Precord <cprecord@editra.org>                                  #
 # Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE: pycomp.py                                                          #
-# AUTHOR: Cody Precord                                                     #
-# LANGUAGE: Python                                                         #
-# SUMMARY:                                                                 #
-#    Provides completion and calltip support for python documents. To      #
-# provide the completion lists and calltips a mix of parsing and           #
-# introspection is used to deduct the requested information.               #
-#                                                                          #
-#--------------------------------------------------------------------------#
+Provides completion and calltip support for python documents. To provide the
+completion lists and calltips a mix of parsing and introspection is used to
+deduct the requested information.
+
+@summary: Python autocompletion support
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: pycomp.py 51011 2008-01-04 13:57:42Z CJP $"
-__revision__ = "$Revision: 51011 $"
+__cvsid__ = "$Id: pycomp.py 56691 2008-11-06 19:26:42Z CJP $"
+__revision__ = "$Revision: 56691 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -49,7 +45,8 @@ class Completer(object):
         self._log = wx.GetApp().GetLog()
         self._buffer = stc_buffer
         self._autocomp_keys = [ord('.')]
-        self._autocomp_stop = ' .,;:([)]}\'"\\<>%^&+-=*/|`'
+        self._autocomp_stop = '\'"\\`'
+        self._autocomp_fillup = ' .,;:([)]}<>%^&+-=*/|'
         self._calltip_keys = [ord('(')]
         self._case_sensitive = False
 
@@ -107,6 +104,11 @@ class Completer(object):
             return self._autocomp_keys
         else:
             return list()
+        
+    def IsAutoCompEvent(self, evt):
+        if evt.ControlDown() and evt.GetKeyCode() == ord(' '):
+            return True
+        return False
 
     def GetAutoCompList(self, command):
         """Returns the list of possible completions for a 
@@ -125,6 +127,9 @@ class Completer(object):
 
         """
         return getattr(self, '_autocomp_stop', u'')
+    
+    def GetAutoCompFillups(self):
+        return getattr(self, '_autocomp_fillup', u'')
 
     def GetCallTip(self, command):
         """Returns the formated calltip string for the command.
@@ -141,6 +146,11 @@ class Completer(object):
 
         """
         return getattr(self, '_calltip_keys', list())
+    
+    def IsCallTipEvent(self, evt):
+        if evt.ControlDown() and evt.GetKeyCode() == ord('9'):
+            return True
+        return False
 
     def GetCaseSensitive(self):
         """Returns whether the autocomp commands are case sensitive
@@ -313,7 +323,7 @@ class PyCompleter(object):
                         if doc is None:
                             doc = max(getattr(result, '__doc__', ' '), ' ')
 
-                        comp = {'word' : meth[len(match):],
+                        comp = {'word' : meth,
                                 'abbr' : meth,
                                 'info' : _cleanstr(str(doc))}
 
