@@ -12,8 +12,8 @@ Editra Control Library: ControlBox
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ctrlbox.py 54782 2008-07-25 00:23:22Z CJP $"
-__revision__ = "$Revision: 54782 $"
+__svnid__ = "$Id: ctrlbox.py 56859 2008-11-20 05:36:31Z CJP $"
+__revision__ = "$Revision: 56859 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -134,6 +134,7 @@ class ControlBar(wx.PyPanel):
         self._style = style
         self._sizer = wx.BoxSizer(wx.HORIZONTAL)
         self._tools = dict(simple=list())
+        self._spacing = (5, 5)
 
         # Setup
         msizer = wx.BoxSizer(wx.VERTICAL)
@@ -172,11 +173,11 @@ class ControlBar(wx.PyPanel):
                 control.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
 
         if align == wx.ALIGN_LEFT:
-            self._sizer.Add((5, 5), 0)
+            self._sizer.Add(self._spacing, 0)
             self._sizer.Add(control, stretch, align|wx.ALIGN_CENTER_VERTICAL)
         else:
             self._sizer.Add(control, stretch, align|wx.ALIGN_CENTER_VERTICAL)
-            self._sizer.Add((5, 5), 0)
+            self._sizer.Add(self._spacing, 0)
 
         self.Layout()
 
@@ -208,7 +209,7 @@ class ControlBar(wx.PyPanel):
             tool.SetMargins(0, 0)
             spacer = (0, 0)
         else:
-            spacer = (5, 5)
+            spacer = self._spacing
         tool.SetToolTipString(help)
 
         self._tools['simple'].append(tool.GetId())
@@ -244,6 +245,14 @@ class ControlBar(wx.PyPanel):
 
         evt.Skip()
 
+    def SetToolSpacing(self, px):
+        """Set the spacing to use between tools/controls.
+        @param px: int (number of pixels)
+        @todo: dynamically update existing layouts
+
+        """
+        self._spacing = (px, px)
+
     def SetVMargin(self, top, bottom):
         """Set the Vertical margin used for spacing controls from the
         top and bottom of the bars edges.
