@@ -61,8 +61,8 @@ Requirements:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: outbuff.py 56205 2008-10-10 02:27:14Z CJP $"
-__revision__ = "$Revision: 56205 $"
+__svnid__ = "$Id: outbuff.py 56777 2008-11-15 00:54:47Z CJP $"
+__revision__ = "$Revision: 56777 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -203,21 +203,20 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         self.SetSelBackground(True, highlight)
         self.__SetupStyles()
 
-    def __PutText(self, txt, ind):
-        """
-        @param txt: String to append
-        @param ind: index in update buffer
+    def __FlushBuffer(self):
+        """Flush the update buffer
+        @postcondition: The update buffer is empty
 
         """
         self._updating.acquire()
         self.SetReadOnly(False)
-        start = self.GetLength() # Get position new text will start from
+        txt = u''.join(self._updates[:])
+        start = self.GetLength()
         self.AppendText(txt)
         self.GotoPos(self.GetLength())
-        self._updates = self._updates[ind:]
+        self._updates = list()
         self.ApplyStyles(start, txt)
         self.SetReadOnly(True)
-        self.RefreshBufferedLines()
         self._updating.release()
 
     def __SetupStyles(self, font=None):
@@ -282,6 +281,21 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         """
         pass
 
+    def CanCopy(self):
+        """Is it possible to copy text right now
+        @return: bool
+
+        """
+        sel = self.GetSelection()
+        return sel[0] != sel[1]
+
+    def CanCut(self):
+        """Is it possible to Cut
+        @return: bool
+
+        """
+        return not self.GetReadOnly()
+
     def Clear(self):
         """Clear the Buffer"""
         self.SetReadOnly(False)
@@ -378,7 +392,7 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         """
         ind = len(self._updates)
         if ind:
-            wx.CallAfter(self.__PutText, u''.join(self._updates), ind)
+            self.__FlushBuffer()
         elif evt is not None:
             self.DoUpdatesEmpty()
         else:
@@ -628,7 +642,7 @@ class ProcessThread(threading.Thread):
                 ctypes.windll.kernel32.PeekNamedPipe(handle, None, 0, 0,
                                                      ctypes.byref(avail), None)
                 if avail.value > 0:
-                    read = self._proc.stdout.readline()
+                    read = self._proc.stdout.read(4096)
                     if read.endswith(os.linesep):
                         read = read[:-1 * len(os.linesep)] + "\n"
                 else:
@@ -659,10 +673,10 @@ class ProcessThread(threading.Thread):
                     if not select.select([self._proc.stdout], [], [], 1)[0]:
                         return True
 
-                    read = self._proc.stdout.readline()
+                    read = self._proc.stdout.read(4096)
                     if read == '':
                         return False
-                except IOError:
+                except IOError, msg:
                     return False
             finally:
                 if not self._proc.stdout.closed:
