@@ -18,23 +18,26 @@ a gradient using system defined colors.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_cmdbar.py 54630 2008-07-14 23:38:58Z CJP $"
-__revision__ = "$Revision: 54630 $"
+__svnid__ = "$Id: ed_cmdbar.py 56476 2008-10-20 17:55:34Z CJP $"
+__revision__ = "$Revision: 56476 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import os
 import sys
 import cStringIO, zlib
 import glob
 import re
 import wx
+
+# Local Imports
 import util
 import ed_glob
 import ed_search
 import ed_event
 import ed_msg
 import eclib.platebtn as platebtn
+import eclib.finddlg as finddlg
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
@@ -61,6 +64,7 @@ ID_SEARCH_CTRL = wx.NewId()
 ID_SEARCH_NEXT = wx.NewId()
 ID_SEARCH_PRE = wx.NewId()
 ID_MATCH_CASE = wx.NewId()
+ID_REGEX = wx.NewId()
 ID_LINE_CTRL = wx.NewId()
 ID_CMD_CTRL = wx.NewId()
 
@@ -90,18 +94,12 @@ class CommandBar(wx.Panel):
                             cmd=wx.BoxSizer())
 
         # Install Controls
-        v_sizer = wx.BoxSizer(wx.VERTICAL)
         self._sizers['h_sizer'].Add((8, 8))
-        bstyle = wx.BU_EXACTFIT
-        if wx.Platform == '__WXGTK__':
-            bstyle = wx.NO_BORDER
-
-        self.close_b = wx.BitmapButton(self, ID_CLOSE_BUTTON, \
-                                       XButton.GetBitmap(), style=bstyle)
+        self.close_b = platebtn.PlateButton(self, ID_CLOSE_BUTTON,
+                                            bmp=XButton.GetBitmap(),
+                                            style=platebtn.PB_STYLE_NOBG)
         self._sizers['h_sizer'].Add(self.close_b, 0, wx.ALIGN_CENTER_VERTICAL)
-        self._sizers['h_sizer'].Add((12, 12))
-        v_sizer.Add((2, 2))
-        self._sizers['h_sizer'].Add(v_sizer)
+        self._sizers['h_sizer'].Add((5, 5))
         self.SetSizer(self._sizers['h_sizer'])
         self.SetAutoLayout(True)
 
@@ -220,15 +218,22 @@ class CommandBar(wx.Panel):
 
         match_case = wx.CheckBox(self, ID_MATCH_CASE, _("Match Case"))
         match_case.SetValue(search.IsMatchCase())
+
+        regex_cb = wx.CheckBox(self, ID_REGEX, _("Regex"))
+        regex_cb.SetValue(search.IsRegEx())
+
+        # Use the small size controls on osx
         if wx.Platform == '__WXMAC__':
             t_sizer.Add((5, 5))
-            for win in [f_lbl, match_case, next_btn, pre_btn]:
+            for win in (f_lbl, match_case, next_btn, pre_btn, regex_cb):
                 win.SetFont(wx.SMALL_FONT)
 
         ctrl_sizer.AddMany([(10, 0), (next_btn, 0, wx.ALIGN_CENTER_VERTICAL),
                             ((5, 0)), (pre_btn, 0, wx.ALIGN_CENTER_VERTICAL),
                             ((8, 0)),
-                            (match_case, 0, wx.ALIGN_CENTER_VERTICAL)])
+                            (match_case, 0, wx.ALIGN_CENTER_VERTICAL),
+                            ((5, 5), 0),
+                            (regex_cb, 0, wx.ALIGN_CENTER_VERTICAL)])
 
         t_sizer.Add(ctrl_sizer, 0, wx.ALIGN_CENTER_VERTICAL)
 
@@ -247,15 +252,20 @@ class CommandBar(wx.Panel):
 
         """
         e_id = evt.GetId()
-        if e_id == ID_MATCH_CASE:
+        if e_id in (ID_MATCH_CASE, ID_REGEX):
             ctrl = self.FindWindowById(e_id)
             if ctrl != None:
                 search = self.FindWindowById(ID_SEARCH_CTRL)
+                if e_id == ID_MATCH_CASE:
+                    flag = finddlg.AFR_MATCHCASE
+                else:
+                    flag = finddlg.AFR_REGEX
+
                 if search != None:
                     if ctrl.GetValue():
-                        search.SetSearchFlag(wx.FR_MATCHCASE)
+                        search.SetSearchFlag(flag)
                     else:
-                        search.ClearSearchFlag(wx.FR_MATCHCASE)
+                        search.ClearSearchFlag(flag)
         else:
             evt.Skip()
 
@@ -872,6 +882,10 @@ class LineCtrl(wx.SearchCtrl):
             for child in self.GetChildren():
                 if isinstance(child, wx.TextCtrl):
                     child.SetValidator(util.IntValidator(0, 65535))
+                    child.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
+                    break
+        else:
+            self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
 
         # Event management
         self.Bind(wx.EVT_TEXT_ENTER, self.OnInput)
@@ -895,6 +909,20 @@ class LineCtrl(wx.SearchCtrl):
         doc.SetFocus()
         self.GetParent().Hide()
 
+    def OnKeyUp(self, evt):
+        """Handle keyup events"""
+        if evt.GetEventType() != wx.wxEVT_KEY_UP:
+            evt.Skip()
+            return
+
+        e_key = evt.GetKeyCode()
+        if e_key == wx.WXK_ESCAPE:
+            # TODO change to more safely determine the context
+            # Currently control is only used in command bar
+            self.GetParent().Hide()
+        else:
+            evt.Skip()
+
 #-----------------------------------------------------------------------------#
 # TODO: merge the common parts of these two classes into a single base class
 
