@@ -14,8 +14,8 @@ access point for all common variables in the project.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_glob.py 56611 2008-10-31 03:43:03Z CJP $"
-__revision__ = "$Revision: 56611 $"
+__svnid__ = "$Id: ed_glob.py 56831 2008-11-18 06:05:05Z CJP $"
+__revision__ = "$Revision: 56831 $"
 
 __all__ = [ 'CONFIG', 'SB_INFO', 'VERSION', 'PROG_NAME', 'ID_NEW', 'ID_OPEN',
             'ID_CLOSE', 'ID_CLOSEALL', 'ID_SAVE', 'ID_SAVEAS', 'ID_SAVEALL',
@@ -39,8 +39,7 @@ __all__ = [ 'CONFIG', 'SB_INFO', 'VERSION', 'PROG_NAME', 'ID_NEW', 'ID_OPEN',
             'ID_COMMAND_BAR', 'ID_DOCUMENTATION', 'ID_COMMAND',
             'ID_USE_SOFTTABS', 'ID_DUP_LINE', 'ID_TRANSLATE',
             'I18N_PAGE', 'ID_GOTO_MBRACE', 'ID_HLCARET_LINE', 'ID_SHOW_SB',
-            'ID_REVERT_FILE', 'ID_RELOAD_ENC', 'ID_DOCPROP'
-]
+            'ID_REVERT_FILE', 'ID_RELOAD_ENC', 'ID_DOCPROP', 'ID_PASTE_AFTER']
 
 #---- Project Info ----#
 # The project info was moved to another module so it could be accessed
@@ -61,6 +60,7 @@ import wxcompat
 # Values set when main loads
 CONFIG = {
           'CONFIG_BASE' : None, # Set if config base is in nonstandard location
+          'CONFIG_DIR'  : "",   # Root configration directory
           'CACHE_DIR'   : "",   # Holds temp data about documents
           'PROFILE_DIR' : "",   # User Profile Directory
           'PLUGIN_DIR'  : "",   # User Plugin Dir
@@ -102,6 +102,7 @@ ID_REDO          = wx.ID_REDO
 ID_CUT           = wx.ID_CUT
 ID_COPY          = wx.ID_COPY
 ID_PASTE         = wx.ID_PASTE
+ID_PASTE_AFTER   = wx.NewId()
 ID_SELECTALL     = wx.ID_SELECTALL
 ID_LINE_EDIT     = wx.NewId()
 ID_BOOKMARK      = wx.NewId()
