@@ -16,8 +16,8 @@ highlight text in a StyledTextCtrl.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_search.py 56608 2008-10-31 02:24:14Z CJP $"
-__revision__ = "$Revision: 56608 $"
+__svnid__ = "$Id: ed_search.py 56838 2008-11-18 18:43:23Z CJP $"
+__revision__ = "$Revision: 56838 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -166,15 +166,16 @@ class SearchController:
         else:
             evt.Skip()
 
-    def OnFind(self, evt):
+    def OnFind(self, evt, findnext=False):
         """Do an incremental search in the currently buffer
         @param evt: EVT_FIND, EVT_FIND_NEXT
+        @keyword findnext: force a find next action
 
         """
         data = self.GetData()
 
-        # Find next from menu event
-        if evt.GetEventType() == wx.wxEVT_COMMAND_MENU_SELECTED:
+        # Find next from menu event or called internally by replace
+        if findnext or evt.GetEventType() == wx.wxEVT_COMMAND_MENU_SELECTED:
             evt = finddlg.FindEvent(finddlg.edEVT_FIND_NEXT,
                                     flags=data.GetFlags())
             evt.SetFindString(data.GetFindString())
@@ -221,6 +222,7 @@ class SearchController:
         match = engine.Find(spos)
         if match is not None:
             start, end = match
+
             if isdown:
                 stc.SetSelection(start + spos, end + spos)
             else:
@@ -244,6 +246,7 @@ class SearchController:
 
             if match is not None:
                 start, end = match
+
                 self._posinfo['found'] = start
 
                 match = list(match)
@@ -327,6 +330,9 @@ class SearchController:
         replacestring = evt.GetReplaceString()
         self._stc().ReplaceSelection(replacestring)
 
+        # Go to the next match
+        self.OnFind(None, True)
+
     def OnReplaceAll(self, evt):
         """Replace all instance of the search string with the given
         replace string for the given search context.
@@ -336,21 +342,22 @@ class SearchController:
         rstring = evt.GetReplaceString()
         engine = SearchEngine(evt.GetFindString(), evt.IsRegEx(),
                               True, evt.IsMatchCase(), evt.IsWholeWord())
+        results = 0
 
         if smode == finddlg.LOCATION_CURRENT_DOC:
             stc = self._stc()
             engine.SetSearchPool(stc.GetTextRaw())
             matches = engine.FindAll()
             if matches is not None:
-                self.ReplaceInStc(stc, matches, rstring)
-            # TODO report number of items replaced
+                self.ReplaceInStc(stc, matches, rstring, evt.IsRegEx())
+                results = len(matches)
         elif smode == finddlg.LOCATION_OPEN_DOCS:
             for ctrl in self._parent.GetTextControls():
                 engine.SetSearchPool(ctrl.GetTextRaw())
                 matches = engine.FindAll()
                 if matches is not None:
-                    self.ReplaceInStc(ctrl, matches, rstring)
-            # TODO report number of items replaced
+                    self.ReplaceInStc(ctrl, matches, rstring, evt.IsRegEx())
+                    results += len(matches)
         elif smode == finddlg.LOCATION_IN_FILES:
             dlg = wx.MessageDialog(self._parent,
                                    _("Sorry will be ready for next version"),
@@ -360,6 +367,7 @@ class SearchController:
                                    style=wx.ICON_WARNING|wx.OK|wx.CANCEL|wx.CENTER)
             result = dlg.ShowModal()
             dlg.Destroy()
+
             if result == wx.ID_OK:
                 pass
 #                path = evt.GetDirectory()
@@ -369,6 +377,12 @@ class SearchController:
             else:
                 return
 
+        # Post number of matches that were replaced to the status bar
+        if results > 0:
+            ed_msg.PostMessage(ed_msg.EDMSG_UI_SB_TXT,
+                              (ed_glob.SB_INFO,
+                              _("%d matches were replaced.") % results))
+
     def OnShowFindDlg(self, evt):
         """Catches the Find events and shows the appropriate find dialog
         @param evt: event that called this handler
@@ -398,7 +412,7 @@ class SearchController:
         self._finddlg.SetFocus()
 
     @staticmethod
-    def ReplaceInStc(stc, matches, rstring):
+    def ReplaceInStc(stc, matches, rstring, isregex=True):
         """Replace the strings at the position in the given StyledTextCtrl
         @param stc: StyledTextCtrl
         @param matches: list of tuples [(s1, e1), (s2, e2)]
@@ -406,10 +420,15 @@ class SearchController:
 
         """
         stc.BeginUndoAction()
-        for start, end in reversed(matches):
+        for match in reversed(matches):
+            start, end = match.span()
+            if isregex:
+                value = match.expand(rstring)
+            else:
+                value = rstring
             stc.SetTargetStart(start)
             stc.SetTargetEnd(end)
-            stc.ReplaceTarget(rstring)
+            stc.ReplaceTarget(value)
         stc.EndUndoAction()
 
     def SetLookinChoices(self, choices):
@@ -476,15 +495,15 @@ class SearchEngine:
         """
         tmp = self._query
         if not self._isregex:
-            tmp = EscapeRegEx(tmp)
+            tmp = re.escape(tmp)
 
         if self._wholeword:
-            tmp = "\\s%s\\s" % tmp
+            tmp = "\\b%s\\b" % tmp
 
-        if self._matchcase:
-            flags = 0
-        else:
-            flags = re.IGNORECASE
+        flags = re.MULTILINE
+
+        if not self._matchcase:
+            flags |= re.IGNORECASE
 
         if self._unicode:
             flags |= re.UNICODE
@@ -519,11 +538,8 @@ class SearchEngine:
         if self._regex is None:
             return list()
 
-        matches = [ match for match in re.finditer(self._regex, self._pool) ]
-        if len(matches):
-            matches = [match.span() for match in matches]
-            return matches
-        return list()
+        matches = [match for match in re.finditer(self._regex, self._pool)]
+        return matches
 
     def FindAllLines(self):
         """Find all the matches in the current context
@@ -746,16 +762,6 @@ class SearchEngine:
 
 #-----------------------------------------------------------------------------#
 
-def EscapeRegEx(regex):
-    """Escape all special regex characters in the given string
-    @param regex: string
-    @return: string
-
-    """
-    for char in u"\\[](){}+*$^?":
-        regex = regex.replace(char, "\\%s" % char)
-    return regex
-
 def FormatResult(fname, lnum, match):
     """Format the search result string
     @return: string
@@ -852,7 +858,6 @@ class EdSearchCtrl(wx.SearchCtrl):
 
         if self.GetValue() == self._last:
             s_cmd = finddlg.edEVT_FIND_NEXT
-        self.InsertHistoryItem(self.GetValue())
 
         evt = finddlg.FindEvent(s_cmd, flags=self._flags)
         self._last = self.GetValue()
@@ -1020,11 +1025,10 @@ class EdSearchCtrl(wx.SearchCtrl):
 
         # Dont do search for navigation keys
         if tmp == wx.EmptyString or evt.CmdDown() or \
-           e_key in [wx.WXK_COMMAND, wx.WXK_LEFT, wx.WXK_RIGHT,
-                     wx.WXK_UP, wx.WXK_DOWN, wx.WXK_F1, wx.WXK_F2,
-                     wx.WXK_F3, wx.WXK_F4, wx.WXK_F5, wx.WXK_F6,
-                     wx.WXK_F7, wx.WXK_F8, wx.WXK_F9, wx.WXK_F10,
-                     wx.WXK_F11, wx.WXK_F12]:
+           e_key in [wx.WXK_COMMAND, wx.WXK_LEFT, wx.WXK_RIGHT, wx.WXK_CONTROL,
+                     wx.WXK_ALT, wx.WXK_UP, wx.WXK_DOWN, wx.WXK_F1, wx.WXK_F2, 
+                     wx.WXK_F3, wx.WXK_F4, wx.WXK_F5, wx.WXK_F6, wx.WXK_F7, 
+                     wx.WXK_F8, wx.WXK_F9, wx.WXK_F10, wx.WXK_F11, wx.WXK_F12]:
             return
 
         if e_key == wx.WXK_RETURN or e_key == wx.WXK_F3:
@@ -1032,6 +1036,10 @@ class EdSearchCtrl(wx.SearchCtrl):
                 self.DoSearch(next=False)
             else:
                 self.DoSearch(next=True)
+
+            # Add to search history
+            if e_key == wx.WXK_RETURN:
+                self.InsertHistoryItem(self.GetValue())
         else:
             # Don't do incremental searches when the RegEx flag is set in order
             # to avoid errors in compiling the expression
