@@ -18,8 +18,8 @@ specific options such as commenting code.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 56649 2008-11-02 17:19:34Z CJP $"
-__revision__ = "$Revision: 56649 $"
+__svnid__ = "$Id: ed_stc.py 56950 2008-11-24 01:25:08Z CJP $"
+__revision__ = "$Revision: 56950 $"
 
 #-------------------------------------------------------------------------#
 # Imports
@@ -49,10 +49,17 @@ MARK_MARGIN = 0
 NUM_MARGIN  = 1
 FOLD_MARGIN = 2
 
+# EOL Constants
+EDSTC_EOL_CR   = "CR"
+EDSTC_EOL_LF   = "LF"
+EDSTC_EOL_CRLF = "CRLF"
+
+# Character sets
 SPACECHARS = " \t\r\n"
 NONSPACE = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_"
 OPERATORS = "./\?[]{}<>!@#$%^&*():=-+\"';,"
 
+# Key code additions
 ALT_SHIFT = wx.stc.STC_SCMOD_ALT|wx.stc.STC_SCMOD_SHIFT
 CTRL_SHIFT = wx.stc.STC_SCMOD_CTRL|wx.stc.STC_SCMOD_SHIFT
 
@@ -113,7 +120,6 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         self._code = dict(compsvc=autocomp.AutoCompService(self),
                           synmgr=syntax.SyntaxMgr(ed_glob.CONFIG['CACHE_DIR']),
                           keywords=[ ' ' ],
-                          syntax_set=list(),
                           comment=list(),
                           clexer=None,      # Container lexer method
                           indenter=None,    # Auto indenter
@@ -161,6 +167,9 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
        #---- End Init ----#
 
+    def __del__(self):
+        self.file.RemoveModifiedCallback(self.FireModified)
+
     @property
     def __name__(self):
         return u"EditraTextCtrl"
@@ -399,7 +408,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         self.SetTabWidth(_PGET('TABWIDTH', 'int'))
 #        self.SetTabIndents(True) # Add option for this too?
         self.SetIndentationGuides(_PGET('GUIDES'))
-        self.SetEOLFromString(_PGET('EOL'))
+        self.SetEOLMode(_PGET('EOL'))
         self.SetViewEOL(_PGET('SHOW_EOL'))
         self.SetAutoComplete(_PGET('AUTO_COMP'))
         self.FoldingOnOff(_PGET('CODE_FOLD'))
@@ -463,6 +472,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """Check if copy/cut is possible"""
         return self.GetSelectionStart() != self.GetSelectionEnd()
 
+    CanCut = CanCopy
+
     def ConvertCase(self, upper=False):
         """Converts the case of the selected text to either all lower
         case(default) or all upper case.
@@ -664,26 +675,10 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.SetCurrentPos(self.GetCurrentPos() + 1)
         self.DeleteBack()
 
-    def FindTagById(self, style_id):
-        """Find the style tag that is associated with the given
-        Id. If not found it returns an empty string.
-        @param style_id: id of tag to look for
-        @return: style tag string
-        @todo: change syntax modules to all use ids
-
-        """
-        for data in self._code['syntax_set']:
-            # If its a standard lexer the style id will be stored as
-            # a string. If its a container lexer it is the id.
-            if isinstance(data[0], basestring):
-                s_id = getattr(wx.stc, data[0])
-            else:
-                s_id = data[0]
-
-            if style_id == s_id:
-                return data[1]
-
-        return 'default_style'
+    def FireModified(self):
+        """Fire a modifed event"""
+        self.OnModified(wx.stc.StyledTextEvent(wx.stc.wxEVT_STC_MODIFIED,
+                                               self.GetId()))
 
     def GetAutoComplete(self):
         """Is Autocomplete being used by this instance
@@ -707,39 +702,6 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         return self.file
 
-    def GetStyleSheet(self, sheet_name=None):
-        """Finds the current style sheet and returns its path. The
-        Lookup is done by first looking in the users config directory
-        and if it is not found there it looks for one on the system
-        level and if that fails it returns None.
-        @param sheet_name: style sheet to look for
-        @return: full path to style sheet
-
-        """
-        if sheet_name:
-            style = sheet_name
-            if sheet_name.split(u'.')[-1] != u"ess":
-                style += u".ess"
-        elif _PGET('SYNTHEME', 'str').split(u'.')[-1] != u"ess":
-            style = (_PGET('SYNTHEME', 'str') + u".ess").lower()
-        else:
-            style = _PGET('SYNTHEME', 'str').lower()
-
-        # Get Correct Filename if it exists
-        for sheet in util.GetResourceFiles('styles', False, True, title=False):
-            if sheet.lower() == style.lower():
-                style = sheet
-                break
-
-        user = os.path.join(ed_glob.CONFIG['STYLES_DIR'], style)
-        sysp = os.path.join(ed_glob.CONFIG['SYS_STYLES_DIR'], style)
-        if os.path.exists(user):
-            return user
-        elif os.path.exists(sysp):
-            return sysp
-        else:
-            return None
-
     def OnKeyDown(self, evt):
         """Handles keydown events, currently only deals with
         auto indentation.
@@ -806,6 +768,12 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.AddText(unichr(key_code))
             if self._config['autocomp']:
                 self.ShowAutoCompOpt(command)
+        elif self._code['compsvc'].IsAutoCompEvent(evt):
+            if self.AutoCompActive():
+                self.AutoCompCancel()
+            command = self.GetCommandStr()
+            if self._config['autocomp']:
+                self.ShowAutoCompOpt(command)
         elif key_code in self._code['compsvc'].GetCallTipKeys():
             if self.AutoCompActive():
                 self.AutoCompCancel()
@@ -813,6 +781,12 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.AddText(unichr(key_code))
             if self._config['autocomp']:
                 self.ShowCallTip(command)
+        elif self._code['compsvc'].IsCallTipEvent(evt):
+            if self.AutoCompActive():
+                self.AutoCompCancel()
+            command = self.GetCommandStr()
+            if self._config['autocomp']:
+                self.ShowCallTip(command[:command.rfind('(')])
         else:
             evt.Skip()
         return
@@ -829,7 +803,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
     def PostPositionEvent(self):
         """Post an event to update the status of the line/column"""
-        msg = _("Line: %d  Column: %d") % self.GetPos()
+        pos = self.GetPos()
+        msg = _("Line: %(lnum)d  Column: %(cnum)d") % dict(lnum=pos[0], cnum=pos[1])
         nevt = ed_event.StatusEvent(ed_event.edEVT_STATUS, self.GetId(),
                                     msg, ed_glob.SB_ROWCOL)
         wx.PostEvent(self.GetTopLevelParent(), nevt)
@@ -895,7 +870,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         curr_pos = self.GetCurrentPos()
         start = curr_pos - 1
         col = self.GetColumn(curr_pos)
-        cmd_lmt = list(self._code['compsvc'].GetAutoCompStops())
+        cmd_lmt = list(self._code['compsvc'].GetAutoCompStops() + \
+                       self._code['compsvc'].GetAutoCompFillups())
         for key in self._code['compsvc'].GetAutoCompKeys():
             kval = chr(key)
             if kval in cmd_lmt:
@@ -917,7 +893,8 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         lst = self._code['compsvc'].GetAutoCompList(command)
         if len(lst):
-            self.AutoCompShow(0, u' '.join(lst))
+            pos = self.GetCurrentPos()
+            self.AutoCompShow(pos - self.WordStartPosition(pos, True), u' '.join(lst))
             self.SetFocus()
 
     def ShowCallTip(self, command):
@@ -1213,6 +1190,11 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.SetViewEdgeGuide(not self.GetEdgeMode())
         elif e_id == ed_glob.ID_SHOW_EOL:
             self.SetViewEOL(not self.GetViewEOL())
+        elif e_id == ed_glob.ID_PASTE_AFTER:
+            cpos = self.GetCurrentPos()
+            self.Paste()
+            self.SetCurrentPos(cpos)
+            self.SetSelection(cpos, cpos)
         elif e_id == ed_glob.ID_SHOW_WS:
             self.SetViewWhiteSpace(not self.GetViewWhiteSpace())
         elif e_id == ed_glob.ID_WORD_WRAP:
@@ -1475,18 +1457,18 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         self.file.SetEncoding(enc)
 
-    def SetEOLFromString(self, mode_str):
+    def SetEOLMode(self, mode):
         """Sets the EOL mode from a string descript
         @param mode_str: eol mode to set
-        @todo: get rid of this somehow
+        @note: overrides StyledTextCtrl.SetEOLMode
 
         """
-        mode_map = { 'Macintosh (\\r)' : wx.stc.STC_EOL_CR,
-                     'Unix (\\n)' : wx.stc.STC_EOL_LF,
-                     'Windows (\\r\\n)' : wx.stc.STC_EOL_CRLF
+        mode_map = { EDSTC_EOL_CR   : wx.stc.STC_EOL_CR,
+                     EDSTC_EOL_LF   : wx.stc.STC_EOL_LF,
+                     EDSTC_EOL_CRLF : wx.stc.STC_EOL_CRLF
                    }
-        mode = mode_map.get(mode_str, wx.stc.STC_EOL_LF)
-        self.SetEOLMode(mode)
+        mode = mode_map.get(mode, wx.stc.STC_EOL_LF)
+        wx.stc.StyledTextCtrl.SetEOLMode(self, mode)
 
     def SetFileName(self, path):
         """Set the buffers filename attributes from the given path"""
@@ -1919,8 +1901,7 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         if result:
             self.SetSavePoint()
             self.SetModTime(util.GetFileModTime(path))
-            self.OnModified(wx.stc.StyledTextEvent(wx.stc.wxEVT_STC_MODIFIED,
-                                                   self.GetId()))
+            self.file.FireModified()
             self.SetFileName(path)
 
         wx.CallAfter(ed_msg.PostMessage,
@@ -1954,9 +1935,16 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
         """
         self.AutoCompSetAutoHide(False)
+        self.AutoCompSetChooseSingle(True)
         self._code['compsvc'].LoadCompProvider(self.GetLexer())
         self.AutoCompSetIgnoreCase(self._code['compsvc'].GetIgnoreCase())
         self.AutoCompStops(self._code['compsvc'].GetAutoCompStops())
+        # TODO: come back to this it can cause some annoying behavior where
+        #       it automatically completes strings that you don't want to be
+        #       inserted in the buffer. (i.e typing self._value will bring up
+        #       the autocomp list but if self._value is not in the list and you
+        #       hit space it will automatically insert something from the list.)
+#        self.AutoCompSetFillUps(self._code['compsvc'].GetAutoCompFillups())
 
     def ConfigureLexer(self, file_ext):
         """Sets Lexer and Lexer Keywords for the specifed file extension
@@ -2068,44 +2056,13 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         kwlist = self._code['keywords'].split()    # Split into a list of words
         kwlist = list(set(kwlist))                 # Uniqueify the list
         kwlist.sort()                              # Sort into alphbetical order
+
         # Can't have ? in scintilla autocomp list unless specifying an image
         if '?' in kwlist:
             kwlist.remove('?')
         self._code['keywords'] = " ".join(kwlist)  # Put back into a string
         return True
 
-    def SetSyntax(self, syn_lst):
-        """Sets the Syntax Style Specs from a list of specifications
-        @param syn_lst: [(STYLE_ID, "STYLE_TYPE"), (STYLE_ID2, "STYLE_TYPE2)]
-
-        """
-        # Parses Syntax Specifications list, ignoring all bad values
-        self.UpdateBaseStyles()
-        valid_settings = list()
-        for syn in syn_lst:
-            if len(syn) != 2:
-                self.LOG("[ed_stc][warn] Error setting syntax spec")
-                continue
-            else:
-                if self.GetLexer() == wx.stc.STC_LEX_CONTAINER:
-                    self.StyleSetSpec(syn[0], self.GetStyleByName(syn[1]))
-                else:
-                    if not isinstance(syn[0], basestring) or \
-                       not hasattr(wx.stc, syn[0]):
-                        self.LOG("[ed_stc][warn] Unknown syntax region: %s" % \
-                                 str(syn[0]))
-                        continue
-                    elif not isinstance(syn[1], basestring):
-                        self.LOG("[ed_stc][warn] Poorly formated styletag: %s" % \
-                                 str(syn[1]))
-                        continue
-                    else:
-                        self.StyleSetSpec(getattr(wx.stc, syn[0]), \
-                                          self.GetStyleByName(syn[1]))
-                valid_settings.append(syn)
-        self._code['syntax_set'] = valid_settings
-        return True
-
     def SetProperties(self, prop_lst):
         """Sets the Lexer Properties from a list of specifications
         @param prop_lst: [ ("PROPERTY", "VAL"), ("PROPERTY2", "VAL2) ]
@@ -2134,75 +2091,14 @@ class EditraStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         self.Freeze()
         self.StyleClearAll()
-        self.SetSyntax(self._code['syntax_set'])
+        self.SetSyntax(self.GetSyntaxParams())
         self.DefineMarkers()
         self.Thaw()
         self.Refresh()
 
-    def StyleDefault(self):
-        """Clears the editor styles to default
-        @postcondition: style is reset to default
-
-        """
-        self.StyleClearAll()
-        self.SetCaretForeground(wx.NamedColor("black"))
-        self.Colourise(0, -1)
-
     def UpdateBaseStyles(self):
-        """Updates the base styles of editor to the current settings
-        @postcondition: base style info is updated
-
-        """
-        self.StyleDefault()
-        self.SetMargins(4, 0)
-        # Global default styles for all languages
-        self.StyleSetSpec(0, self.GetStyleByName('default_style'))
-        self.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, \
-                          self.GetStyleByName('default_style'))
-        self.StyleSetSpec(wx.stc.STC_STYLE_LINENUMBER, \
-                          self.GetStyleByName('line_num'))
-        self.StyleSetSpec(wx.stc.STC_STYLE_CONTROLCHAR, \
-                          self.GetStyleByName('ctrl_char'))
-        self.StyleSetSpec(wx.stc.STC_STYLE_BRACELIGHT, \
-                          self.GetStyleByName('brace_good'))
-        self.StyleSetSpec(wx.stc.STC_STYLE_BRACEBAD, \
-                          self.GetStyleByName('brace_bad'))
-        self.StyleSetSpec(wx.stc.STC_STYLE_INDENTGUIDE, \
-                          self.GetStyleByName('guide_style'))
-
-        # wx.stc.STC_STYLE_CALLTIP doesnt seem to do anything
-        calltip = self.GetItemByName('calltip')
-        self.CallTipSetBackground(calltip.GetBack())
-        self.CallTipSetForeground(calltip.GetFore())
-
-        sback = self.GetItemByName('select_style')
-        if not sback.IsNull():
-            sback = sback.GetBack()
-        else:
-            sback = wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT)
-        self.SetSelBackground(True, sback)
-
-        wspace = self.GetItemByName('whitespace_style')
-        if not wspace.IsNull():
-            self.SetWhitespaceBackground(True, wspace.GetBack())
-            self.SetWhitespaceForeground(True, wspace.GetFore())
-
-        self.SetCaretForeground(self.GetDefaultForeColour())
-        self.SetCaretLineBack(self.GetItemByName('caret_line').GetBack())
+        ed_style.StyleMgr.UpdateBaseStyles(self)
         self.DefineMarkers()
-        self.Colourise(0, -1)
-
-    def UpdateAllStyles(self, spec_style=None):
-        """Refreshes all the styles and attributes of the control
-        @param spec_style: style scheme name
-        @postcondition: style scheme is set to specified style
-
-        """
-        if spec_style != self.style_set:
-            self.LoadStyleSheet(self.GetStyleSheet(spec_style), force=True)
-        self.UpdateBaseStyles()
-        self.SetSyntax(self._code['syntax_set'])
-        self.Refresh()
 
     #---- End Style Definitions ----#
 
