@@ -16,13 +16,15 @@ and setting of the program by setting values in the Profile.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: prefdlg.py 54780 2008-07-24 02:56:08Z CJP $"
-__revision__ = "$Revision: 54780 $"
+__svnid__ = "$Id: prefdlg.py 56626 2008-11-01 02:18:01Z CJP $"
+__revision__ = "$Revision: 56626 $"
 
 #----------------------------------------------------------------------------#
 # Dependancies
 import wx
 import wx.lib.mixins.listctrl as listmix
+import locale
+import encodings
 import os
 import sys
 
@@ -110,6 +112,8 @@ class PreferencesDialog(wx.Frame):
         self.SetTransparent(Profile_Get('ALPHA', 'int', 255))
 
         # Attributes
+        self._accel = wx.AcceleratorTable([(wx.ACCEL_NORMAL, wx.WXK_ESCAPE, wx.ID_CLOSE)])
+        self.SetAcceleratorTable(self._accel)
         self._tbook = PrefTools(self)
         sizer = wx.BoxSizer(wx.VERTICAL)
 
@@ -121,6 +125,7 @@ class PreferencesDialog(wx.Frame):
         wx.GetApp().RegisterWindow(repr(self), self, True)
 
         # Bind Events
+        self.Bind(wx.EVT_MENU, lambda evt: self.Close(), id=wx.ID_CLOSE)
         self.Bind(wx.EVT_CLOSE, self.OnClose)
         self.Bind(wx.EVT_SHOW, self.OnShow)
 
@@ -219,9 +224,12 @@ class PrefTools(wx.Toolbook):
             width = psz.GetWidth()
         else:
             width = tbw
+
+        page.Freeze()
         parent.SetClientSize((width, psz.GetHeight() + tbh))
         parent.SendSizeEvent()
         parent.Layout()
+        page.Thaw()
         if evt is not None:
             evt.Skip()
 
@@ -257,53 +265,7 @@ class PrefTools(wx.Toolbook):
 
 #-----------------------------------------------------------------------------#
 
-class PrefPanelBase(wx.Panel):
-    """Base of all preference panels
-    @summary: Provides a panel with a painted background
-
-    """
-    def __init__(self, parent, style=wx.TAB_TRAVERSAL | wx.SUNKEN_BORDER):
-        """Default Constructor
-        @param parent: The panels parent
-
-        """
-        wx.Panel.__init__(self, parent, style=style)
-
-        # Event Handlers
-        if wx.Platform == '__WXMAC__':
-            self.Bind(wx.EVT_PAINT, self.OnPaint)
-
-    def OnPaint(self, evt):
-        """Paints the panels background
-        @param evt: Event that called this handler
-
-        """
-        # This is odd but I have recieved a number of error reports where
-        # wx is being reported as None in this function requires more
-        # investigation as I cannot reproduce
-        if 'wx' not in globals() or wx is None:
-            evt.Skip()
-            return
-
-        dc = wx.PaintDC(self)
-        gc = wx.GraphicsContext.Create(dc)
-        col1 = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
-        brush = gc.CreateBrush(wx.Brush(util.AdjustColour(col1, 5)))
-        rect = self.GetRect()
-
-        # Create the background path
-        path = gc.CreatePath()
-        path.AddRectangle(0, 0, rect.width, rect.height)
-
-        gc.SetPen(wx.Pen(util.AdjustColour(col1, -10), 1))
-        gc.SetBrush(brush)
-        gc.DrawPath(path)
-
-        evt.Skip()
-
-#-----------------------------------------------------------------------------#
-
-class GeneralPanel(PrefPanelBase):
+class GeneralPanel(wx.Panel):
     """Creates a panel with controls for Editra's general settings
     @summary: Panel with a number of controls that affect the users
               global profile setting for how Editra should operate.
@@ -314,13 +276,15 @@ class GeneralPanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
 
         # Attributes
         self.SetToolTipString(_("Changes made in this dialog are saved in your "
                                 "current profile. Some Items such as Language "
                                 "require the program to be restarted before "
                                 "taking effect."))
+
+        # Layout
         self.__DoLayout()
 
         # Event Handlers
@@ -346,8 +310,7 @@ class GeneralPanel(PrefPanelBase):
         psizer.AddMany([(wx.StaticText(self, label=_("Printer Mode") + u": "),
                          0, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
                         (ExChoice(self, ed_glob.ID_PRINT_MODE,
-                                  choices=['Black/White', 'Colour/White',
-                                         'Colour/Default', 'Inverse', 'Normal'],
+                                  choices=GetPrintModeStrings(),
                                   default=Profile_Get('PRINT_MODE')),
                          0, wx.ALIGN_CENTER_VERTICAL)])
 
@@ -444,20 +407,22 @@ class GeneralPanel(PrefPanelBase):
         @note: Also handles the Language ComboBox
 
         """
-        util.Log("[prefldg][evt] General Page: Choice event caught")
         e_id = evt.GetId()
         e_obj = evt.GetEventObject()
-        if e_id in [ed_glob.ID_PREF_MODE, ed_glob.ID_PRINT_MODE,
-                    ed_glob.ID_PREF_FHIST, ed_glob.ID_PREF_LANG]:
+        if e_id in [ed_glob.ID_PREF_MODE,
+                    ed_glob.ID_PREF_FHIST,
+                    ed_glob.ID_PREF_LANG]:
             Profile_Set(ed_glob.ID_2_PROF[e_id], e_obj.GetValue())
             if e_id == ed_glob.ID_PREF_MODE:
                 ed_glob.DEBUG = ('DEBUG' in e_obj.GetValue())
+        elif e_id == ed_glob.ID_PRINT_MODE:
+            Profile_Set(ed_glob.ID_2_PROF[e_id], e_obj.GetSelection())
         else:
             evt.Skip()
 
 #-----------------------------------------------------------------------------#
 
-class DocumentPanel(PrefPanelBase):
+class DocumentPanel(wx.Panel):
     """Creates a panel with controls for Editra's editing settings
     @summary: Contains a wx.Notebook that contains a number of pages with
               setting controls for how documents are handled by the
@@ -469,7 +434,7 @@ class DocumentPanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
 
         # Attributes
         self._DoLayout()
@@ -490,6 +455,14 @@ class DocumentPanel(PrefPanelBase):
         msizer.AddMany([(sizer, 1, wx.EXPAND), ((10, 10), 0)])
         self.SetSizer(msizer)
 
+    def GetSize(self):
+        """Get the size of the panel
+        @return: wx.Size
+
+        """
+        sz = wx.Panel.GetSize(self)
+        return wx.Size(sz[0] + 35, sz[1])
+
 class DocGenPanel(wx.Panel):
     """Panel used for general document settings in the DocumentPanel's
     notebook.
@@ -538,6 +511,9 @@ class DocGenPanel(wx.Panel):
                           default=Profile_Get('INDENTWIDTH', 'str')), 0,
                         wx.ALIGN_CENTER_VERTICAL)])
 
+        at_cb = wx.CheckBox(self, ed_glob.ID_PREF_AUTOTRIM,
+                            _("Automatically trim whitespace on save"))
+        at_cb.SetValue(Profile_Get('AUTO_TRIM_WS', 'bool', False))
         ut_cb = wx.CheckBox(self, ed_glob.ID_PREF_TABS,
                             _("Use Tabs Instead of Spaces"))
         ut_cb.SetValue(Profile_Get('USETABS', 'bool', False))
@@ -554,6 +530,21 @@ class DocGenPanel(wx.Panel):
                                  default=Profile_Get('EOL')),
                         0, wx.ALIGN_CENTER_VERTICAL)])
 
+        # Encoding options
+        d_encoding = Profile_Get('ENCODING',
+                                 'str',
+                                 default=locale.getpreferredencoding())
+        if d_encoding is None:
+            d_encoding = 'utf-8'
+            Profile_Set('ENCODING', d_encoding)
+
+        d_encoding = encodings.normalize_encoding(d_encoding)
+
+        enc_ch = ExChoice(self, ed_glob.ID_PREF_ENCODING,
+                          choices=util.GetAllEncodings(),
+                          default=d_encoding)
+        enc_ch.SetToolTipString(_("Encoding to try when auto detection fails"))
+
         # View Options
         aa_cb = wx.CheckBox(self, ed_glob.ID_PREF_AALIAS, _("AntiAliasing"))
         aa_cb.SetValue(Profile_Get('AALIASING'))
@@ -581,16 +572,20 @@ class DocGenPanel(wx.Panel):
                                   "regions when syntax highlighting is in use"))
 
         # Layout
-        sizer = wx.FlexGridSizer(18, 2, 5, 5)
+        sizer = wx.FlexGridSizer(21, 2, 5, 5)
         sizer.AddGrowableCol(1, 1)
         sizer.AddMany([((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("Format") + u": "),
-                        0, wx.ALIGN_CENTER_VERTICAL), (ut_cb, 0),
+                        0, wx.ALIGN_CENTER_VERTICAL), (at_cb, 0),
+                       ((5, 5), 0), (ut_cb, 0),
                        ((5, 5), 0), (bsu_cb, 0),
                        ((5, 5), 0), (tabsz, 0),
                        ((5, 5), 0), (indentsz, 0),
                        ((5, 5), 0), (eolsz, 0),
                        ((10, 10), 0), ((10, 10), 0),
+                       (wx.StaticText(self, label=_("Prefered Encoding") + u":"),
+                        0), (enc_ch, 1),
+                       ((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("View Options") + u": "),
                         0), (aa_cb, 0),
                        ((5, 5), 0), (seol_cb, 0),
@@ -652,10 +647,14 @@ class DocGenPanel(wx.Panel):
                     ed_glob.ID_PREF_AALIAS, ed_glob.ID_SHOW_EOL,
                     ed_glob.ID_SHOW_LN, ed_glob.ID_SHOW_WS,
                     ed_glob.ID_WORD_WRAP, ed_glob.ID_PREF_AALIAS,
-                    ed_glob.ID_PREF_INDENTW]:
+                    ed_glob.ID_PREF_INDENTW, ed_glob.ID_PREF_ENCODING,
+                    ed_glob.ID_PREF_AUTOTRIM ]:
             Profile_Set(ed_glob.ID_2_PROF[e_id],
                         evt.GetEventObject().GetValue())
-            wx.CallLater(25, DoUpdates)
+
+            # Do updates for everything but text encoding
+            if e_id not in (ed_glob.ID_PREF_ENCODING, ed_glob.ID_PREF_AUTOTRIM):
+                wx.CallLater(25, DoUpdates)
         else:
             evt.Skip()
 
@@ -798,7 +797,6 @@ class DocSyntaxPanel(wx.Panel):
 
         # Attributes
         self._elist = ExtListCtrl(self)
-        self._elist.LoadList()
         self._elist.SetMinSize((425, 200))
 
         # Layout page
@@ -893,7 +891,7 @@ class DocSyntaxPanel(wx.Panel):
 
 #-----------------------------------------------------------------------------#
 
-class AppearancePanel(PrefPanelBase):
+class AppearancePanel(wx.Panel):
     """Creates a panel with controls for Editra's appearance settings
     @summary: contains all the controls for configuring the appearance
               related settings in Editra.
@@ -904,7 +902,7 @@ class AppearancePanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
 
         # Layout
         self._DoLayout()
@@ -1081,11 +1079,11 @@ class AppearancePanel(PrefPanelBase):
 
 #-----------------------------------------------------------------------------#
 
-class NetworkPanel(PrefPanelBase):
+class NetworkPanel(wx.Panel):
     """Network related configration options"""
     def __init__(self, parent):
         """Create the panel"""
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
 
         # Layout
         self.__DoLayout()
@@ -1347,7 +1345,7 @@ class UpdatePage(wx.Panel):
 #-----------------------------------------------------------------------------#
 # Advanced Page
 
-class AdvancedPanel(PrefPanelBase):
+class AdvancedPanel(wx.Panel):
     """Creates a panel for holding advanced configuration options
     @summary: Contains a wx.Notebook that contains a number of pages with
               setting controls for configuring the advanced configuration
@@ -1359,7 +1357,7 @@ class AdvancedPanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
 
         # Layout
         self.__DoLayout()
@@ -1389,13 +1387,13 @@ ID_KEYS = wx.NewId()
 ID_BINDING = wx.NewId()
 KEYS = ['', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
         'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
-        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '+', 'Left',
-        'Right', 'Down', 'Up', 'Home', 'End', 'Tab']
+        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '+', '/', ',',
+        '.', '[', ']', '{', '}', ':', 'Left', 'Right', 'Down', 'Up', 'Home',
+        'End', 'Tab']
 KEYS.extend(["F" + str(x) for x in range(1, 13)])
 
 if wx.Platform == '__WXMSW__':
     KEYS.remove('Tab')
-KEYS.sort()
 
 MODIFIERS = ['', 'Alt', 'Shift']
 if wx.Platform == '__WXMAC__':
@@ -1789,7 +1787,6 @@ class ExtListCtrl(wx.ListCtrl,
                                    wx.LC_VRULES | wx.BORDER)
 
         listmix.ListCtrlAutoWidthMixin.__init__(self)
-        listmix.TextEditMixin.__init__(self)
         elistmix.ListRowHighlighter.__init__(self)
 
         # Setup
@@ -1798,6 +1795,8 @@ class ExtListCtrl(wx.ListCtrl,
                           _("Extensions (space separated, no dots)"))
         self._extreg = syntax.ExtensionRegister()
         self._editing = None
+        self.LoadList()
+        listmix.TextEditMixin.__init__(self)
 
     def CloseEditor(self, evt=None):
         """Update list and extension register after edit window
@@ -1877,8 +1876,10 @@ class ExChoice(wx.Choice):
             for ind in range(len(choices)):
                 choices[ind] = str(choices[ind])
         wx.Choice.__init__(self, parent, cid, choices=choices)
-        if default != None:
+        if default != None and isinstance(default, basestring):
             self.SetStringSelection(default)
+        elif default is not None:
+            self.SetSelection(default)
 
     def GetValue(self):
         """Gets the Selected Value
@@ -1986,6 +1987,18 @@ class PyFontPicker(wx.Panel):
 #-----------------------------------------------------------------------------#
 # Utility Functions
 
+def GetPrintModeStrings():
+    """Get the strings for describing the print modes
+    @note: defined in a function so translations can take place at runtime
+    @note: Order must be kept in sync with the PRINT_ vals in ed_glob
+
+    """
+    return [_('Black/White'),       # PRINT_BLACK_WHITE
+            _('Colour/White'),      # PRINT_COLOR_WHITE
+            _('Colour/Default'),    # PRINT_COLOR_DEF
+            _('Inverse'),           # PRINT_INVERSE
+            _('Normal')]            # PRINT_NORMAL
+
 def DoUpdates():
     """Update all open text controls"""
     for mainw in wx.GetApp().GetMainWindows():
