@@ -17,8 +17,8 @@ editor can load to configure the styles of the text.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: style_editor.py 54990 2008-08-05 23:37:20Z CJP $"
-__revision__ = "$Revision: 54990 $"
+__svnid__ = "$Id: style_editor.py 56787 2008-11-15 23:37:01Z CJP $"
+__revision__ = "$Revision: 56787 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -271,8 +271,8 @@ class StyleEditor(wx.Dialog):
 
         """
         if ed_glob.CONFIG['STYLES_DIR'] == ed_glob.CONFIG['SYS_STYLES_DIR']:
-            user_config = os.path.join(wx.GetHomeDir(),
-                                        "." + ed_glob.PROG_NAME, 'styles')
+            stdpath = wx.StandardPaths_Get()
+            user_config = os.path.join(stdpath.GetUserDataDir(), 'styles')
             if not os.path.exists(user_config):
                 try:
                     os.mkdir(user_config)
@@ -336,6 +336,9 @@ class StyleEditor(wx.Dialog):
             stage1 = wx.EmptyString
             for attr in ('fore', 'back', 'face', 'size'):
                 ival = item.GetNamedAttr(attr)
+                if attr in ('fore', 'back'):
+                    ival = ival.upper()
+
                 if ival is None or ival == ditem.GetNamedAttr(attr):
                     continue
 
