@@ -34,8 +34,8 @@ objects such as the Extension Register.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: syntax.py 56177 2008-10-09 02:03:21Z CJP $"
-__revision__ = "$Revision: 56177 $"
+__svnid__ = "$Id: syntax.py 56759 2008-11-14 00:47:06Z CJP $"
+__revision__ = "$Revision: 56759 $"
 
 #-----------------------------------------------------------------------------#
 # Dependencies
@@ -239,6 +239,9 @@ def GenFileFilters():
     f_dict = dict()
     for key, val in extreg.iteritems():
         val.sort()
+        if key.lower() == 'makefile':
+            continue
+
         f_dict[key] = u";*." + u";*.".join(val)
 
     # Build the final list of properly formated strings
