@@ -12,14 +12,17 @@ This file contains various helper functions and utilities that the program uses.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: util.py 54760 2008-07-21 23:37:55Z CJP $"
-__revision__ = "$Revision: 54760 $"
+__svnid__ = "$Id: util.py 56941 2008-11-23 17:20:07Z CJP $"
+__revision__ = "$Revision: 56941 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import os
 import sys
 import stat
+import types
+import mimetypes
+import encodings
 import codecs
 import urllib2
 import wx
@@ -29,6 +32,8 @@ import ed_glob
 import ed_event
 import ed_crypt
 import dev_tool
+import syntax.syntax as syntax
+import syntax.synglob as synglob
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
@@ -141,9 +146,7 @@ class DropTargetFT(wx.PyDropTarget):
             point = wx.Point(x_cord, y_cord)
             self._tmp.BeginDrag(point - self._lastp, stc)
             self._tmp.Hide()
-            stc.GotoPos(stc.PositionFromPoint(point))
-            stc.Refresh()
-            stc.Update()
+            stc.DoDragOver(x_cord, y_cord, drag_result)
             self._tmp.Move(point)
             self._tmp.Show()
             self._tmp.RedrawImage(self._lastp, point, True, True)
@@ -164,8 +167,8 @@ class DropTargetFT(wx.PyDropTarget):
         except wx.PyAssertionError:
             wx.PostEvent(self.window.GetTopLevelParent(), \
                         ed_event.StatusEvent(ed_event.edEVT_STATUS, -1,
-                                             _("Unable to open dropped file or "
-                                               "text")))
+                                             _("Unable to accept dropped file "
+                                               "or text")))
             data = False
             drag_result = wx.DragCancel
 
@@ -174,16 +177,8 @@ class DropTargetFT(wx.PyDropTarget):
             text = self._data['tdata'].GetText()
             if len(files) > 0 and self._data['fcallb'] is not None:
                 self._data['fcallb'](files)
-            elif(len(text) > 0):
-                if hasattr(self.window, 'PositionFromPointClose'):
-                    pos = self.window.PositionFromPointClose(x_cord, y_cord)
-                else:
-                    pos = wx.stc.STC_INVALID_POSITION
-
-                if pos != wx.stc.STC_INVALID_POSITION:
-                    self.window.InsertText(pos, text)
-                else:
-                    drag_result = wx.DragCancel
+            elif len(text) > 0:
+                self.window.DoDropText(x_cord, y_cord, text)
         self.InitObjects()
         return drag_result
 
@@ -268,16 +263,9 @@ def FilterFiles(file_list):
 
     """
     good = list()
+    checker = FileTypeChecker()
     for path in file_list:
-        # Filter out directories and some common filetypes that can't
-        # be opened.
-        if not os.path.exists(path) or os.path.isdir(path) or \
-           GetExtension(path).lower() in ['gz', 'tar', 'bz2', 'zip', 'rar',
-                                          'ace', 'png', 'jpg', 'gif', 'jpeg',
-                                          'bmp', 'exe', 'pyc', 'pyo', 'psd',
-                                          'a', 'o', 'dll', 'ico', 'icns']:
-            continue
-        else:
+        if not checker.IsBinary(path):
             good.append(path)
     return good
 
@@ -292,6 +280,28 @@ def GetFileModTime(file_name):
         mod_time = 0
     return mod_time
 
+def GetFileType(fname):
+    """Get what the type of the file is as Editra sees it
+    in a formatted string.
+    @param fname: file path
+    @return: string (formatted/translated filetype)
+
+    """
+    if os.path.isdir(fname):
+        return _("Folder")
+
+    eguess = syntax.GetTypeFromExt(fname.split('.')[-1])
+    if eguess == synglob.LANG_TXT and fname.split('.')[-1] == 'txt':
+        return _("Text Document")
+    elif eguess == synglob.LANG_TXT:
+        mtype = mimetypes.guess_type(fname)[0]
+        if mtype is not None:
+            return mtype
+        else:
+            return _("Unknown")
+    else:
+        return _("%s Source File") % eguess
+
 def GetFileReader(file_name, enc='utf-8'):
     """Returns a file stream reader object for reading the
     supplied file name. It returns a file reader using the encoding
@@ -329,7 +339,7 @@ def GetFileWriter(file_name, enc='utf-8'):
 
     """
     try:
-        file_h = file(file_name, "wb")
+        file_h = open(file_name, "wb")
     except IOError:
         dev_tool.DEBUGP("[file_writer][err] Failed to open file %s" % file_name)
         return -1
@@ -458,51 +468,30 @@ def MakeNewFolder(path, name):
 
     return (True, folder)
 
-def ResolvAbsPath(rel_path):
-    """Takes a relative path and converts it to an
-    absolute path.
-    @param rel_path: path to construct absolute path for
-
-    """
-    cwd = os.getcwd()
-    pieces = rel_path.split(os.sep)
-    cut = 0
-
-    for token in pieces:
-        if token == "..":
-            cut += 1
-
-        if cut > 0:
-            rpath = os.sep.join(pieces[cut:])
-            cut *= -1
-            cwd = cwd.split(os.sep)
-            apath = os.sep.join(cwd[0:cut])
-        else:
-            rpath = rel_path
-            apath = cwd
-
-    return apath + os.sep + rpath
-
 def HasConfigDir(loc=u""):
     """ Checks if the user has a config directory and returns True
     if the config directory exists or False if it does not.
     @return: whether config dir in question exists on an expected path
 
     """
-    if os.path.exists(u"%s%s.%s%s%s" % (wx.GetHomeDir(), os.sep,
-                                        ed_glob.PROG_NAME, os.sep, loc)):
-        return True
-    else:
-        return False
+    cbase = ed_glob.CONFIG['CONFIG_BASE']
+    if cbase is None:
+        cbase = wx.StandardPaths_Get().GetUserDataDir() + os.sep
+
+    to_check = os.path.join(cbase, loc)
+    return os.path.exists(to_check)
 
 def MakeConfigDir(name):
     """Makes a user config directory
     @param name: name of config directory to make in user config dir
 
     """
-    config_dir = wx.GetHomeDir() + os.sep + u"." + ed_glob.PROG_NAME
+    cbase = ed_glob.CONFIG['CONFIG_BASE']
+    if cbase is None:
+        cbase = wx.StandardPaths_Get().GetUserDataDir()
+
     try:
-        os.mkdir(config_dir + os.sep + name)
+        os.mkdir(cbase + os.sep + name)
     except (OSError, IOError):
         pass
 
@@ -513,9 +502,12 @@ def CreateConfigDir():
 
     """
     #---- Resolve Paths ----#
-    config_dir = u"%s%s.%s" % (wx.GetHomeDir(), os.sep, ed_glob.PROG_NAME)
-    profile_dir = u"%s%sprofiles" % (config_dir, os.sep)
-    dest_file = u"%s%sdefault.ppb" % (profile_dir, os.sep)
+    config_dir = ed_glob.CONFIG['CONFIG_BASE']
+    if config_dir is None:
+        config_dir = wx.StandardPaths_Get().GetUserDataDir() + os.sep
+
+    profile_dir = os.path.join(config_dir, u"profiles")
+    dest_file = os.path.join(profile_dir, u"default.ppb")
     ext_cfg = ["cache", "styles", "plugins"]
 
     #---- Create Directories ----#
@@ -547,11 +539,16 @@ def ResolvConfigDir(config_dir, sys_only=False):
            the code has proven itself.
 
     """
+    stdpath = wx.StandardPaths_Get()
+
+    # Try to get a User config directory
     if not sys_only:
-        # Try to look for a user dir
-        user_config = u"%s%s.%s%s%s" % (wx.GetHomeDir(), os.sep,
-                                        ed_glob.PROG_NAME, os.sep,
-                                        config_dir)
+        user_config = ed_glob.CONFIG['CONFIG_BASE']
+        if user_config is None:
+            user_config = stdpath.GetUserDataDir() + os.sep
+
+        user_config = os.path.join(user_config, config_dir)
+
         if os.path.exists(user_config):
             return user_config + os.sep
 
@@ -562,6 +559,8 @@ def ResolvConfigDir(config_dir, sys_only=False):
     path = os.sep.join(path.split(os.sep)[:-2])
     path =  path + os.sep + config_dir + os.sep
     if os.path.exists(path):
+        if not isinstance(path, types.UnicodeType):
+            path = unicode(path, sys.getfilesystemencoding())
         return path
 
     # If we get here we need to do some platform dependant lookup
@@ -586,7 +585,11 @@ def ResolvConfigDir(config_dir, sys_only=False):
             pieces = pro_path.split(os.sep)
             pro_path = os.sep.join(pieces[:-1])
         else:
-            pro_path = ResolvAbsPath(pro_path)
+            pro_path = os.path.abspath(pro_path)
+    elif os.sys.platform == "darwin":
+        # On OS X the config directories are in the applet under Resources
+        pro_path = stdpath.GetResourcesDir()
+        pro_path = os.path.join(pro_path, config_dir)
     else:
         pro_path = os.sep.join(pieces[:-2])
 
@@ -598,16 +601,17 @@ def ResolvConfigDir(config_dir, sys_only=False):
             if pieces[-1] not in [ed_glob.PROG_NAME.lower(), ed_glob.PROG_NAME]:
                 pro_path = os.sep.join(pieces[:-1])
         else:
-            pro_path = ResolvAbsPath(pro_path)
+            pro_path = os.path.abspath(pro_path)
 
-    if os.sys.platform == "darwin":
-        # On OS X the config directories are in the applet under Resources
-        pro_path = u"%s%sResources%s%s%s" % (pro_path, os.sep, os.sep,
-                                             config_dir, os.sep)
-    else:
+    if os.sys.platform != "darwin":
         pro_path = pro_path + os.sep + config_dir + os.sep
 
-    return os.path.normpath(pro_path) + os.sep
+    path = os.path.normpath(pro_path) + os.sep
+    # Make sure path is unicode
+    if not isinstance(path, types.UnicodeType):
+        path = unicode(path, sys.getdefaultencoding())
+
+    return path
 
 def GetResources(resource):
     """Returns a list of resource directories from a given toplevel config dir
@@ -673,6 +677,17 @@ def GetResourceFiles(resource, trim=True, get_all=False,
         
         return list(set(rec_list))
 
+def GetAllEncodings():
+    """Get all encodings found on the system
+    @return: list of strings
+
+    """
+    elist = encodings.aliases.aliases.values()
+    elist = list(set(elist))
+    elist.sort()
+    elist = [ enc for enc in elist if not enc.endswith('codec') ]
+    return elist
+
 def Log(msg):
     """Push the message to the apps log
     @param msg: message string to log
@@ -765,6 +780,59 @@ def SetWindowIcon(window):
 
 #-----------------------------------------------------------------------------#
 
+class FileTypeChecker:
+    """File type checker and recognizer"""
+    TXTCHARS = ''.join(map(chr, [7,8,9,10,12,13,27] + range(0x20, 0x100)))
+    ALLBYTES = ''.join(map(chr, range(256)))
+
+    def __init__(self, preread=4096):
+        """Create the FileTypeChecker
+        @keyword preread: number of bytes to read for checking file type
+
+        """
+        # Attributes
+        self._preread = preread
+
+    def _GetHandle(self, fname):
+        """Get a file handle for reading
+        @param fname: filename
+        @return: file object or None
+
+        """
+        try:
+            handle = open(fname, 'rb')
+        except:
+            handle = None
+        return handle
+
+    def IsBinary(self, fname):
+        """Is the file made up of binary data
+        @param fname: filename to check
+        @return: bool
+
+        """
+        handle = self._GetHandle(fname)
+        if handle is not None:
+            bytes = handle.read(self._preread)
+            handle.close()
+            nontext = bytes.translate(FileTypeChecker.ALLBYTES,
+                                      FileTypeChecker.TXTCHARS)
+            return bool(nontext)
+        else:
+            return False
+
+    def IsReadableText(self, fname):
+        """Is the given path readable as text
+        @param fname: filename
+
+        """
+        f_ok = False
+        if os.access(fname, os.R_OK):
+            f_ok = not self.IsBinary(fname)
+        return f_ok
+
+#-----------------------------------------------------------------------------#
+
 class IntValidator(wx.PyValidator):
     """A Generic integer validator"""
     def __init__(self, min_=0, max_=0):
